import { FileData } from './models/file-data.interface';
/**
 * Client provides abilities to create and get bitcoinfiles
 */
export declare class Client {
    options: {
        bitdb_api_base: string;
        bitdb_api_key: string;
        bitcoinfiles_api_base: string;
    };
    constructor(options: any);
    /**
     * Resolve a promise and/or invoke a callback
     * @param resolveOrReject Resolve or reject function to call when done
     * @param data Data to pass forward
     * @param callback Invoke an optional callback first
     */
    private callbackAndResolve;
    private hexEncode;
    private isUtf8;
    /**
     *
     * @param request create request
     * @param callback Optional callback to invoke after completed
     */
    create(request: {
        file: FileData;
        pay: {
            key: string;
        };
        signatures: Array<{
            key: string;
        }>;
    }, callback?: Function): Promise<any>;
    /**
     *
     * @param request create request
     * @param callback Optional callback to invoke after completed
     */
    datapay(request: {
        data: any[];
        pay: {
            key: string;
        };
    }, callback?: Function): Promise<any>;
    /**
     * Hex encode data. Also lowercase any hex data
     * @param data data to hex encode if it's not already hex
     */
    hexEncodeIfNeeded(data: string | undefined): string;
    /**
     * Builds the file and returns the parameters to send to datapay
     *
     * @param request create request
     * @param callback Optional callback to invoke after completed
     */
    buildFile(request: {
        file: FileData;
        signatures?: Array<{
            key: string;
        }>;
    }, callback?: Function): Promise<any>;
    /**
     * Get a file by txid
     * @param txid txid of file
     * @param callback Optional callback to invoke after completed
     */
    get(txid: string, callback?: Function): Promise<any>;
    /**
     * Find files matching certain criteria
     * @param request find request
     * @param callback Optional callback to invoke after completed
     */
    find(request: {
        address?: string;
        contentType?: string;
        name?: string;
        tags?: string[];
        skip?: number;
        limit?: number;
        sort?: any;
        debug?: boolean;
    }, callback?: Function): Promise<any>;
    private baseQuery;
    private addFindClause;
}
