"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const datapay = require("datapay");
const axios_1 = require("axios");
const textEncoder = require("text-encoder");
const utils_1 = require("./utils");
const defaultOptions = {
    bitdb_api_base: 'https://babel.bitdb.network/q/1DHDifPvtPgKFPZMRSxmVHhiPvFmxZwbfh/',
    bitdb_api_key: '12cHytySdrQGRtuvvkVde2j3e74rmEn5TM',
    bitcoinfiles_api_base: 'https://media.bitcoinfiles.org',
};
/**
 * Client provides abilities to create and get bitcoinfiles
 */
class Client {
    constructor(options) {
        this.options = defaultOptions;
        this.options = Object.assign({}, this.options, options);
    }
    /**
     * Resolve a promise and/or invoke a callback
     * @param resolveOrReject Resolve or reject function to call when done
     * @param data Data to pass forward
     * @param callback Invoke an optional callback first
     */
    callbackAndResolve(resolveOrReject, data, callback) {
        if (callback) {
            callback(data);
        }
        if (resolveOrReject) {
            return resolveOrReject(data);
        }
    }
    hexEncode(str) {
        function buf2hex(buffer) {
            const hexStr = Array.prototype.map.call(new Uint8Array(buffer), (x) => ('00' + x.toString(16)).slice(-2)).join('');
            return hexStr.toLowerCase();
        }
        const checkHexPrefixRegex = /^0x(.*)/i;
        const match = checkHexPrefixRegex.exec(str);
        if (match) {
            return str;
        }
        else {
            let enc = new textEncoder.TextEncoder().encode(str);
            return buf2hex(enc);
        }
    }
    isUtf8(encoding) {
        if (!encoding || /\s*/i.test(encoding)) {
            return true;
        }
        return /utf\-?8$/i.test(encoding);
    }
    /**
     *
     * @param request create request
     * @param callback Optional callback to invoke after completed
     */
    create(request, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!request.pay || !request.pay.key || request.pay.key === '') {
                return new Promise((resolve) => {
                    return this.callbackAndResolve(resolve, {
                        success: false,
                        message: 'key required'
                    }, callback);
                });
            }
            const buildResult = yield this.buildFile(request);
            if (!buildResult.success) {
                return new Promise((resolve) => {
                    this.callbackAndResolve(resolve, {
                        success: false,
                        message: buildResult.message
                    }, callback);
                });
            }
            const newArgs = [];
            for (const i of buildResult.data) {
                const checkHexPrefixRegex = /^0x(.*)/i;
                const match = checkHexPrefixRegex.exec(i);
                if (match && match[1]) {
                    newArgs.push(i);
                }
                else {
                    newArgs.push('0x' + i);
                }
            }
            return this.datapay({
                data: newArgs,
                pay: request.pay,
            }, callback);
        });
    }
    /**
     *
     * @param request create request
     * @param callback Optional callback to invoke after completed
     */
    datapay(request, callback) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            datapay.send({
                data: request.data,
                pay: {
                    key: request.pay.key,
                }
            }, (err, transaction) => __awaiter(this, void 0, void 0, function* () {
                if (err) {
                    console.log('err', err);
                    return this.callbackAndResolve(resolve, {
                        success: false,
                        message: err.message ? err.message : err.toString()
                    }, callback);
                }
                return this.callbackAndResolve(resolve, {
                    success: true,
                    txid: transaction
                }, callback);
            }));
        }));
    }
    /**
     * Hex encode data. Also lowercase any hex data
     * @param data data to hex encode if it's not already hex
     */
    hexEncodeIfNeeded(data) {
        if (!data) {
            return '0x00';
        }
        const checkHexPrefixRegex = /^0x(.*)/i;
        const match = checkHexPrefixRegex.exec(data);
        if (match && match[1]) {
            return data.toLowerCase();
        }
        return '0x' + this.hexEncode(data).toLowerCase();
    }
    /**
     * Builds the file and returns the parameters to send to datapay
     *
     * @param request create request
     * @param callback Optional callback to invoke after completed
     */
    buildFile(request, callback) {
        return new Promise((resolve, reject) => {
            if (!request.file) {
                return this.callbackAndResolve(resolve, {
                    success: false,
                    message: 'file required'
                }, callback);
            }
            if (!request.file.content) {
                return this.callbackAndResolve(resolve, {
                    success: false,
                    message: 'content required'
                }, callback);
            }
            if (!request.file.contentType) {
                return this.callbackAndResolve(resolve, {
                    success: false,
                    message: 'contentType required'
                }, callback);
            }
            try {
                let encoding = request.file.encoding ? request.file.encoding : 'utf-8';
                if (this.isUtf8(encoding)) {
                    encoding = 'utf-8';
                }
                let args = [
                    '0x' + Buffer.from("19HxigV4QyBv3tHpQVcUEQyq1pzZVdoAut").toString('hex'),
                    this.hexEncodeIfNeeded(request.file.content),
                    this.hexEncodeIfNeeded(request.file.contentType),
                    this.hexEncodeIfNeeded(encoding)
                ];
                const hasFileName = request.file.name && request.file.name !== '';
                let filename = request.file.name ? request.file.name : '0x00';
                args.push(this.hexEncodeIfNeeded(filename));
                if (request.file.tags) {
                    request.file.tags.map((tag) => args.push(this.hexEncodeIfNeeded(tag)));
                }
                // Attach signatures if they are provided
                if (request.signatures && Array.isArray(request.signatures)) {
                    for (const signatureKey of request.signatures) {
                        if (!signatureKey.key || /^\s*$/.test(signatureKey.key)) {
                            return this.callbackAndResolve(resolve, {
                                success: false,
                                message: 'signature key required'
                            }, callback);
                        }
                        const identityPrivateKey = new datapay.bsv.PrivateKey(signatureKey.key);
                        const identityAddress = identityPrivateKey.toAddress().toLegacyAddress();
                        args.push('0x' + Buffer.from('|').toString('hex'));
                        const opReturnHexArray = utils_1.Utils.buildAuthorIdentity({
                            args: args,
                            address: identityAddress,
                            key: signatureKey.key
                        });
                        args = args.concat(opReturnHexArray);
                    }
                }
                return this.callbackAndResolve(resolve, {
                    success: true,
                    data: args,
                }, callback);
            }
            catch (ex) {
                console.log('ex', ex);
                this.callbackAndResolve(resolve, {
                    success: false,
                    message: ex.message ? ex.message : ex.toString()
                }, callback);
            }
        });
    }
    /**
     * Get a file by txid
     * @param txid txid of file
     * @param callback Optional callback to invoke after completed
     */
    get(txid, callback) {
        return new Promise((resolve, reject) => {
            if (!txid) {
                return this.callbackAndResolve(resolve, {
                    success: false,
                    message: 'txid required'
                }, callback);
            }
            try {
                let query = this.baseQuery({ 'blk.i': -1 }, 0, 0);
                query = this.addFindClause(query, 'tx.h', txid);
                axios_1.default.get(this.options.bitdb_api_base + Buffer.from(JSON.stringify(query)).toString('base64'), {
                    headers: { key: this.options.bitdb_api_key }
                }).then((response) => {
                    let tx = response.data.u[0] ? response.data.u[0] : response.data.c[0] ? response.data.c[0] : undefined;
                    if (!tx) {
                        throw Error('not found');
                    }
                    const formattedResults = [];
                    formattedResults.push({
                        txid: tx.txid,
                        url: `${this.options.bitcoinfiles_api_base}/${tx.txid}`
                    });
                    this.callbackAndResolve(resolve, {
                        success: true,
                        data: formattedResults
                    }, callback);
                }).catch((ex) => {
                    console.log('ex', ex);
                    this.callbackAndResolve(resolve, {
                        success: false,
                        message: ex.message ? ex.message : ex.toString()
                    }, callback);
                });
            }
            catch (ex) {
                console.log('ex', ex.toString(), ex);
                this.callbackAndResolve(resolve, {
                    success: false,
                    message: ex.message ? ex.message : ex.toString()
                }, callback);
            }
        });
    }
    /**
     * Find files matching certain criteria
     * @param request find request
     * @param callback Optional callback to invoke after completed
     */
    find(request, callback) {
        return new Promise((resolve, reject) => {
            if (!request) {
                return this.callbackAndResolve(resolve, {
                    success: false,
                    message: 'field required'
                }, callback);
            }
            try {
                let limit = request.limit ? request.limit : 1;
                let skip = request.skip ? request.skip : 0;
                let sort = request.sort ? request.sort : { 'blk.i': -1 };
                let query = this.baseQuery(sort, limit, skip);
                if (request.address) {
                    query = this.addFindClause(query, 'in.e.a', request.address);
                }
                if (request.contentType) {
                    query = this.addFindClause(query, 'out.s3', request.contentType);
                }
                if (request.name) {
                    query = this.addFindClause(query, 'out.s5', request.name);
                }
                if (request.tags && request.tags.length) {
                    let tagStartIndex = 6;
                    for (const tag of request.tags) {
                        // Do not search on null or undefined tag, skip it
                        if (tag === undefined || tag === null) {
                            tagStartIndex++;
                            continue;
                        }
                        // If the value is prefixed with '0x' then we know it's hex
                        // Otherwise treat it as a string
                        const checkHexPrefixRegex = /^0x(.*)/i;
                        let tagField = 'out.s';
                        let tagValue = tag;
                        const match = checkHexPrefixRegex.exec(tag);
                        if (match && match[1]) {
                            tagField = 'out.h';
                            tagValue = match[1];
                        }
                        tagField += tagStartIndex;
                        query = this.addFindClause(query, tagField, tagValue);
                        tagStartIndex++;
                    }
                }
                if (request.debug) {
                    console.log('find query', query);
                }
                axios_1.default.get(this.options.bitdb_api_base + Buffer.from(JSON.stringify(query)).toString('base64'), {
                    headers: { key: this.options.bitdb_api_key }
                }).then((response) => {
                    let unconfirmed = response.data.u;
                    let confirmed = response.data.c;
                    let txs = [];
                    if (unconfirmed.length) {
                        txs = unconfirmed;
                    }
                    if (confirmed.length) {
                        txs = txs.concat(confirmed);
                    }
                    const formattedResults = [];
                    txs.map((tx) => {
                        formattedResults.push({
                            txid: tx.txid,
                            url: `${this.options.bitcoinfiles_api_base}/${tx.txid}`
                        });
                    });
                    this.callbackAndResolve(resolve, {
                        success: true,
                        data: formattedResults
                    }, callback);
                }).catch((ex) => {
                    console.log('ex', ex);
                    this.callbackAndResolve(resolve, {
                        success: false,
                        message: ex.message ? ex.message : ex.toString()
                    }, callback);
                });
            }
            catch (ex) {
                console.log('ex', ex.toString(), ex);
                this.callbackAndResolve(resolve, {
                    success: false,
                    message: ex.message ? ex.message : ex.toString()
                }, callback);
            }
        });
    }
    baseQuery(sort, limit = 1, skip = 0) {
        return {
            "v": 3,
            "q": {
                "find": {},
                "limit": limit,
                "skip": skip,
                "sort": sort
            },
            "r": {
                "f": "[.[] | { txid: .tx.h, inputInfo: . | { in: .in? }, blockInfo: . | { blockIndex: .blk.i?, blockTime: .blk.t?}, out: .out  } ]"
            }
        };
    }
    addFindClause(query, field, value) {
        const updatedQuery = query;
        updatedQuery.q.find = Object.assign({}, updatedQuery.q.find, {
            [field]: value
        });
        return updatedQuery;
    }
}
exports.Client = Client;
